@@ -60,7 +60,8 @@ update_scan_devices_cb (Scanner *scanner, GList *devices)
                                /* Warning displayed when no scanners are detected */
                                _("No scanners detected"),
                                /* Hint to user on why there are no scanners detected */
-                               _("Please check your scanner is connected and powered on"));
+                               _("Please check your scanner is connected and powered on"),
+			       FALSE);
             g_free (selected);
         }
     }
@@ -159,7 +160,8 @@ scanner_failed_cb (Scanner *scanner, GError *error)
     ui_show_error (ui,
                    /* Title of error dialog when scan failed */
                    _("Failed to scan"),
-                   error->message);
+                   error->message,
+		   TRUE);
 }
 
 
@@ -236,47 +238,44 @@ page_removed_cb (Book *book, Page *page, SimpleScan *ui)
 }
 
 
+static gboolean
+save_book (const gchar *uri, GError **error)
+{
+    gboolean result;
+    gchar *uri_lower;
+
+    uri_lower = g_utf8_strdown (uri, -1);
+    if (g_str_has_suffix (uri_lower, ".pdf"))
+        result = book_save_pdf (book, uri, error);
+    else if (g_str_has_suffix (uri_lower, ".ps"))
+        result = book_save_ps (book, uri, error);
+    else if (g_str_has_suffix (uri_lower, ".png"))
+        result = book_save_png (book, uri, error);
+    else
+        result = book_save_jpeg (book, uri, error);
+
+    g_free (uri_lower);
+
+    return result;
+}
+
+
 static void
-save_cb (SimpleScan *ui, gchar *uri)
+save_cb (SimpleScan *ui, const gchar *uri)
 {
-    GFile *file;
     GError *error = NULL;
-    GFileOutputStream *stream;
 
-    file = g_file_new_for_uri (uri);
+    g_debug ("Saving to '%s'", uri);
 
-    stream = g_file_replace (file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error);
-    if (!stream) {
+    if (!save_book (uri, &error)) {
         g_warning ("Error saving file: %s", error->message);
+        ui_show_error (ui,
+                       /* Title of error dialog when save failed */
+                       _("Failed to save file"),
+                       error->message,
+		       FALSE);
         g_error_free (error);
     }
-    else {
-        gboolean result;
-        gchar *uri_lower;
-
-        uri_lower = g_utf8_strdown (uri, -1);
-        if (g_str_has_suffix (uri_lower, ".pdf"))
-            result = book_save_pdf (book, stream, &error);
-        else if (g_str_has_suffix (uri_lower, ".ps"))
-            result = book_save_ps (book, stream, &error);
-        else if (g_str_has_suffix (uri_lower, ".png"))
-            result = book_save_png (book, stream, &error);
-        else
-            result = book_save_jpeg (book, stream, &error);
-
-        g_free (uri_lower);           
-
-        if (error) {
-            g_warning ("Error saving file: %s", error->message);
-            ui_show_error (ui,
-                           /* Title of error dialog when save failed */
-                           _("Failed to save file"),
-                           error->message);
-            g_error_free (error);
-        }
-
-        g_output_stream_close (G_OUTPUT_STREAM (stream), NULL, NULL);
-    }
 }
 
 
@@ -284,10 +283,7 @@ static void
 email_cb (SimpleScan *ui)
 {
     gint i;
-    GFileOutputStream *stream = NULL;
-    GError *error = NULL;
-    gchar *dir, *path = NULL;
-    GString *filename;
+    gchar *dir, *path = NULL, *uri = NULL;
 
     // TODO: Delete old files on startup
 
@@ -296,8 +292,8 @@ email_cb (SimpleScan *ui)
     g_mkdir_with_parents (dir, 0700);
 
     for (i = 0; ; i++) {
-        GFile *file;
-        gboolean done = TRUE;
+        GString *filename;
+        GError *error = NULL;
 
         filename = g_string_new ("");
         g_string_printf (filename, "scan-%d-%d.pdf", getpid (), i);
@@ -305,30 +301,11 @@ email_cb (SimpleScan *ui)
         g_free (path);
         path = g_build_filename (dir, filename->str, NULL);
         g_string_free (filename, TRUE);
-        file = g_file_new_for_path (path);
-        stream = g_file_create (file, G_FILE_CREATE_NONE, NULL, &error);
-        g_object_unref (file);
 
-        if (!stream) {
-	    done = FALSE;
+        g_free (uri);
+        uri = g_filename_to_uri (path, NULL, NULL);
 
-            if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
-	        g_warning ("Unable to save email PDF: %s", error->message);
-                done = TRUE;
-            }
-
-            g_clear_error (&error);
-            stream = NULL;
-        }
-
-        if (done)
-            break;
-    }
-
-    g_free (dir);
-
-    if (stream) {
-        if (book_save_pdf (book, stream, &error)) {
+        if (book_save_pdf (book, uri, &error)) {
             GString *command_line;
 
             command_line = g_string_new ("");
@@ -341,14 +318,20 @@ email_cb (SimpleScan *ui)
                 g_clear_error (&error);
             }
             g_string_free (command_line, TRUE);
+	    break;
         }
         else {
-            g_warning ("Unable to save email PDF: %s", error->message);	   
-            g_clear_error (&error);
+            if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+                g_warning ("Unable to save email PDF: %s", error->message);
+                g_clear_error (&error);
+                break;
+            }
         }
-
-        g_object_unref (stream);
     }
+
+    g_free (path);
+    g_free (uri);   
+    g_free (dir);
 }
 
 
@@ -382,6 +365,7 @@ usage(int show_gtk)
     fprintf(stderr,
             /* Description on how to use simple-scan displayed on command-line */    
             _("Help Options:\n"
+              "  -d, --debug                     Print debugging messages\n"	      
               "  -v, --version                   Show release version\n"
               "  -h, --help                      Show help options\n"
               "  --help-all                      Show all help options\n"
@@ -405,15 +389,30 @@ usage(int show_gtk)
 }
 
 
+static void
+log_cb (const gchar *log_domain, GLogLevelFlags log_level,
+	const gchar *message, gpointer data)
+{
+    if (log_level & G_LOG_LEVEL_DEBUG)
+       return;
+    g_log_default_handler (log_domain, log_level, message, data);
+}
+
+
 static void
 get_options (int argc, char **argv)
 {
     int i;
+    gboolean debug = FALSE;
 
     for (i = 1; i < argc; i++) {
         char *arg = argv[i];
 
-        if (strcmp (arg, "-v") == 0 ||
+        if (strcmp (arg, "-d") == 0 ||
+            strcmp (arg, "--debug") == 0) {
+            debug = TRUE;
+        }
+        else if (strcmp (arg, "-v") == 0 ||
             strcmp (arg, "--version") == 0) {
             version ();
             exit (0);
@@ -423,7 +422,8 @@ get_options (int argc, char **argv)
             usage (FALSE);
             exit (0);
         }
-        else if (strcmp (arg, "--help-all") == 0) {
+        else if (strcmp (arg, "--help-all") == 0 ||
+                 strcmp (arg, "--help-gtk") == 0) {
             usage (TRUE);
             exit (0);
         }
@@ -435,15 +435,22 @@ get_options (int argc, char **argv)
             default_device = arg;
         }
     }
+   
+    if (!debug)
+        g_log_set_default_handler (log_cb, NULL);
 }
 
 
 int
 main(int argc, char **argv)
 {
+    bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
+    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+    textdomain (GETTEXT_PACKAGE);
+   
     g_thread_init (NULL);
     gtk_init (&argc, &argv);
-    
+
     get_options (argc, argv);
 
     book = book_new ();
